@@ -1,34 +1,26 @@
-#
-# This file is part of MooX-Options
-#
-# This software is copyright (c) 2013 by celogeek <me@celogeek.com>.
-#
-# This is free software; you can redistribute it and/or modify it under
-# the same terms as the Perl 5 programming language system itself.
-#
 package MooX::Options;
 
-# ABSTRACT: Explicit Options eXtension for Object Class
-
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 
-our $VERSION = '4.023';    # VERSION
+our $VERSION = "4.100";
 
-use Locale::TextDomain 'MooX-Options';
+use Carp ('croak');
+use Module::Runtime qw(use_module);
 
 my @OPTIONS_ATTRIBUTES
-    = qw/format short repeatable negativable autosplit autorange doc long_doc order json hidden spacer_before spacer_after/;
+    = qw/format short repeatable negatable autosplit autorange doc long_doc order json hidden spacer_before spacer_after/;
 
 sub import {
     my ( undef, @import ) = @_;
     my $options_config = {
-        protect_argv          => 1,
-        flavour               => [],
-        skip_options          => [],
-        prefer_commandline    => 0,
-        with_config_from_file => 0,
-        usage_string          => undef,
+        protect_argv              => 1,
+        flavour                   => [],
+        skip_options              => [],
+        prefer_commandline        => 0,
+        with_config_from_file     => 0,
+        with_locale_textdomain_oo => 0,
+        usage_string              => undef,
 
         #long description (manual)
         description => undef,
@@ -41,11 +33,8 @@ sub import {
     my $target = caller;
     for my $needed_methods (qw/with around has/) {
         next if $target->can($needed_methods);
-        croak(
-            __x("Can't find the method <{needed_methods}> in <{target}> ! Ensure to load a Role::Tiny compatible module like Moo or Moose before using MooX::Options.",
-                'needed_methods' => $needed_methods,
-                'target'         => $target
-            )
+        croak(    "Can't find the method <$needed_methods> in <$target>!\n"
+                . "Ensure to load a Role::Tiny compatible module like Moo or Moose before using MooX::Options."
         );
     }
 
@@ -58,10 +47,10 @@ sub import {
 
     if (@target_isa) {    #only in the main class, not a role
 
-        use warnings FATAL => 'redefine';
         ## no critic (ProhibitStringyEval, ErrorHandling::RequireCheckingReturnValueOfEval, ValuesAndExpressions::ProhibitImplicitNewlines)
         eval '{
         package ' . $target . ';
+        use MRO::Compat ();
 
             sub _options_data {
                 my ( $class, @meta ) = @_;
@@ -75,7 +64,6 @@ sub import {
 
         1;
         }';
-        use warnings FATAL => qw/void/;
 
         croak($@) if $@;
 
@@ -91,8 +79,7 @@ sub import {
     else {
         if ( $options_config->{with_config_from_file} ) {
             croak(
-                __( "Please, don\'t use the option <with_config_from_file> into a role."
-                )
+                "Please, don't use the option <with_config_from_file> into a role."
             );
         }
     }
@@ -117,6 +104,13 @@ sub import {
         if ( $options_config->{with_config_from_file} ) {
             $with->('MooX::ConfigFromFile::Role');
         }
+        if ( $options_config->{with_locale_textdomain_oo} ) {
+            $with->('MooX::Locale::TextDomain::OO');
+            use_module("MooX::Options::Descriptive::Usage");
+            MooX::Options::Descriptive::Usage->can("localizer")
+                or MooX::Options::Descriptive::Usage->can("with")
+                ->("MooX::Locale::TextDomain::OO");
+        }
 
         $around->(
             _options_data => sub {
@@ -136,14 +130,15 @@ sub import {
         my ( $name, %attributes ) = @_;
         for my $ban (@banish_keywords) {
             croak(
-                __x("You cannot use an option with the name '{ban}', it is implied by MooX::Options",
-                    ban => $ban
-                )
+                "You cannot use an option with the name '$ban', it is implied by MooX::Options"
             ) if $name eq $ban;
         }
 
-        $has->( $name => _filter_attributes(%attributes) );
+        my %_moo_attrs = _filter_attributes(%attributes);
+        $has->( $name => %_moo_attrs ) if %_moo_attrs;
 
+        ## no critic (RegularExpressions::RequireExtendedFormatting)
+        $name =~ s/^\+//;    # one enhances an attribute being an option
         $options_data->{$name}
             = { _validate_and_filter_options(%attributes) };
 
@@ -162,9 +157,10 @@ sub import {
     return;
 }
 
+my %filter_key = map { $_ => 1 } ( @OPTIONS_ATTRIBUTES, 'negativable' );
+
 sub _filter_attributes {
     my %attributes = @_;
-    my %filter_key = map { $_ => 1 } @OPTIONS_ATTRIBUTES;
     return map { ( $_ => $attributes{$_} ) }
         grep { !exists $filter_key{$_} } keys %attributes;
 }
@@ -173,8 +169,6 @@ sub _validate_and_filter_options {
     my (%options) = @_;
     $options{doc} = $options{documentation} if !defined $options{doc};
     $options{order} = 0 if !defined $options{order};
-    $options{autosplit} = ','
-        if !defined $options{autosplit} && $options{autorange};
 
     if ( $options{json}
         || ( defined $options{format} && $options{format} eq 'json' ) )
@@ -183,56 +177,48 @@ sub _validate_and_filter_options {
         delete $options{autosplit};
         delete $options{autorange};
         delete $options{negativable};
+        delete $options{negatable};
         $options{json}   = 1;
         $options{format} = 's';
     }
 
+    if ( $options{autorange} and not defined $options{autosplit} ) {
+
+# XXX maybe we should warn here since a previously beloved feature isn't enabled automatically
+        eval { use_module("Data::Record"); use_module("Regexp::Common"); }
+            and $options{autosplit} = ',';
+    }
+
+    exists $options{negativable}
+        and $options{negatable} = delete $options{negativable};
+
     my %cmdline_options = map { ( $_ => $options{$_} ) }
         grep { exists $options{$_} } @OPTIONS_ATTRIBUTES, 'required';
 
-    $cmdline_options{repeatable} = 1 if $cmdline_options{autosplit};
+    $cmdline_options{repeatable} = 1
+        if $cmdline_options{autosplit} or $cmdline_options{autorange};
     $cmdline_options{format} .= "@"
         if $cmdline_options{repeatable}
         && defined $cmdline_options{format}
         && substr( $cmdline_options{format}, -1 ) ne '@';
 
     croak(
-        __( "Negativable params is not usable with non boolean value, don't pass format to use it !"
+        "Negatable params is not usable with non boolean value, don't pass format to use it !"
         )
-        )
-        if $cmdline_options{negativable} && defined $cmdline_options{format};
+        if ( $cmdline_options{negatable} )
+        and defined $cmdline_options{format};
 
     return %cmdline_options;
 }
 
-sub croak {
-    require Carp;
-    goto &Carp::croak;
-}
-
 1;
 
 __END__
 
-=pod
-
 =head1 NAME
 
 MooX::Options - Explicit Options eXtension for Object Class
 
-=head1 VERSION
-
-version 4.023
-
-=head1 DESCRIPTION
-
-Create a command line tool with your L<Mo>, L<Moo>, L<Moose> objects.
-
-Everything is explicit. You have an C<option> keyword to replace the usual C<has> to explicitly use your attribute into the command line.
-
-The C<option> keyword takes additional parameters and uses L<Getopt::Long::Descriptive>
-to generate a command line tool.
-
 =head1 SYNOPSIS
 
 In myOptions.pm :
@@ -287,11 +273,151 @@ The manual :
 
   perl myTool.pl --man
 
-=head1 METHODS
+=head1 DESCRIPTION
 
-=head2 croak
+Create a command line tool with your L<Mo>, L<Moo>, L<Moose> objects.
 
-Call Carp::croak dynamically
+Everything is explicit. You have an C<option> keyword to replace the usual C<has> to explicitly use your attribute into the command line.
+
+The C<option> keyword takes additional parameters and uses L<Getopt::Long::Descriptive>
+to generate a command line tool.
+
+=head1 IMPORTANT CHANGES IN 4.100
+
+=head2 Enhancing existing attributes
+
+One can now convert an existing attribute into an option for obvious reasons.
+
+  package CommonRole;
+
+  use Moo::Role;
+
+  has attr => (is => "ro", ...);
+
+  sub common_logic { ... }
+
+  1;
+
+  package Suitable::Cmd::CLI;
+
+  use Moo;
+  use MooX::Cmd;
+  use MooX::Options;
+
+  with "CommonRole";
+
+  option '+attr' => (format => 's', repeatable => 1);
+
+  sub execute { shift->common_logic }
+
+  1;
+
+  package Suitable::Web::Request::Handler;
+
+  use Moo;
+
+  with "CommonRole";
+
+  sub all_suits { shift->common_logic }
+
+  1;
+
+  package Suitable::Web;
+
+  use Dancer2;
+  use Suitable::Web::Request::Handler;
+
+  set serializer => "JSON";
+
+  get '/suits' => sub {
+      $my $reqh = Suitable::Web::Request::Handler->new( attr => config->{suit_attr} );
+      $reqh->all_suits;
+  };
+
+  dance;
+
+  1;
+
+Of course there more ways to to it, L<Jedi> or L<Catalyst> shall be fine, either.
+
+=head2 Rename negativable into negatable
+
+Since users stated that C<negativable> is not a reasonable word, the flag is
+renamed into negatable. Those who will 2020 continue use negativable might
+or might not be warned about soon depreciation.
+
+=head2 Replace Locale::TextDomain by MooX::Locale::Passthrough
+
+L<Locale::TextDomain> is broken (technically and functionally) and causes a
+lot of people to avoid C<MooX::Options> or hack around. Both is unintened.
+
+So introduce L<MooX::Locale::Passthrough> to allow any vendor to add reasonable
+localization, eg. by composing L<MooX::Locale::TextDomain::OO> into it's
+solution and initialize the localization in a reasonable way.
+
+=head2 Make lazy loaded features optional
+
+Since some features aren't used on a regular basis, their dependencies have
+been downgraded to C<recommended> or C<suggested>. The optional features are:
+
+=over 4
+
+=item autosplit
+
+This feature allowes one to split option arguments at a defined character and
+always return an array (implicit flag C<repeatable>).
+
+  option "search_path" => ( is => "ro", required => 1, autosplit => ":", format => "s" );
+
+However, this feature requires following modules are provided:
+
+=over 4
+
+=item *
+
+L<Data::Record>
+
+=item *
+
+L<Regexp::Common>
+
+=back
+
+=item json format
+
+This feature allowes one to invoke a script like
+
+  $ my-tool --json-attr '{ "gem": "sapphire", "color": "blue" }'
+
+It might be a reasonable enhancement to I<handles>.
+
+Handling JSON formatted arguments requires any of those modules
+are loded:
+
+=over 4
+
+=item *
+
+L<JSON::MaybeXS>
+
+=item *
+
+L<JSON::PP> (in Core since 5.14).
+
+=back
+
+=back
+
+=head2 Decouple autorange and autosplit
+
+Until 4.023, any option which had autorange enabled got autosplit enabled, too.
+Since autosplit might not work correctly and for a reasonable amount of users
+the fact of
+
+  $ my-tool --range 1..5
+
+is all they desire, autosplit will enabled only when the dependencies of
+autosplit are fulfilled.
 
 =head1 IMPORTED METHODS
 
@@ -383,7 +509,7 @@ This parameter will give the command line an higher priority.
 
 =head2 with_config_from_file
 
-This parameter will load L<MooX::ConfigFromFile> in your module. 
+This parameter will load L<MooX::Options> in your module. 
 The config option will be used between the command line and parameters.
 
 myTool :
@@ -394,6 +520,17 @@ In /etc/myTool.json
 
   {"test" : 1}
 
+=head2 with_locale_textdomain_oo
+
+This Parameter will load L<MooX::Locale::TextDomain::OO> into your module as
+well as into L<MooX::Options::Descriptive::Usage>.
+
+No further action is taken, no language is chosen - everything keep in
+control.
+
+Please read L<Locale::TextDomain::OO> carefully how to enable the desired
+translation setup accordingly.
+
 =head1 usage_string
 
 This parameter is passed to Getopt::Long::Descriptive::describe_options() as
@@ -483,15 +620,17 @@ You can also use the json format
 
   myTool --hash='{"a":1,"b":2}' # hash = { a => 1, b => 2 }
 
-=head2 negativable
+=head2 negatable
 
 It adds the negative version for the option.
 
-  option 'verbose' => (is => 'ro', negativable => 1);
+  option 'verbose' => (is => 'ro', negatable => 1);
 
   myTool --verbose    # verbose = 1
   myTool --no-verbose # verbose = 0
 
+The former name of this flag, negativable, is discouraged - since it's not a word.
+
 =head2 repeatable
 
 It appends to the L</format> the array attribute C<@>.
@@ -511,7 +650,7 @@ For repeatable option, you can add the autosplit feature with your specific para
   
   myTool --test=1 --test=2 # test = (1, 2)
   myTool --test=1,2,3      # test = (1, 2, 3)
-
+  
 It will also handle quoted params with the autosplit.
 
   option testStr => (is => 'ro', format => 's@', default => sub {[]}, autosplit => ',');
@@ -528,7 +667,7 @@ allows you to pass number ranges instead of passing each individual number.
   myTool --test=1 --test=2 # test = (1, 2)
   myTool --test=1,2,3      # test = (1, 2, 3)
   myTool --test=1,2,3..6   # test = (1, 2, 3, 4, 5, 6)
-
+  
 It will also handle quoted params like C<autosplit>, and will not rangify them.
 
   option testStr => (is => 'ro', format => 's@', default => sub {[]}, autorange => 1);
@@ -657,7 +796,7 @@ to use it a lot in L<DuckDuckGo|http://duckduckgo.com> (go to see L<MooX> module
 
 =item * Jens Rehsack (REHSACK)
 
-Use with L<PkgSrc|http://www.pkgsrc.org/>, and many really good idea (L<MooX::Cmd>, L<MooX::ConfigFromFile>, and more to come I'm sure)
+Use with L<PkgSrc|http://www.pkgsrc.org/>, and many really good idea (L<MooX::Cmd>, L<MooX::Options>, and more to come I'm sure)
 
 =item * All contributors
 
@@ -665,14 +804,33 @@ For improving and add more feature to MooX::Options
 
 =back
 
-=head1 BUGS
+=head1 SUPPORT
 
-Please report any bugs or feature requests on the bugtracker website
-https://github.com/celogeek/MooX-Options/issues
+You can find documentation for this module with the perldoc command.
 
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
+    perldoc MooX::Options
+
+You can also look for information at:
+
+=over 4
+
+=item * RT: CPAN's request tracker (report bugs here)
+
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=MooX-Options>
+
+=item * AnnoCPAN: Annotated CPAN documentation
+
+L<http://annocpan.org/dist/MooX-Options>
+
+=item * CPAN Ratings
+
+L<http://cpanratings.perl.org/d/MooX-Options>
+
+=item * Search CPAN
+
+L<http://search.cpan.org/dist/MooX-Options/>
+
+=back
 
 =head1 AUTHOR
 
@@ -682,7 +840,8 @@ celogeek <me@celogeek.com>
 
 This software is copyright (c) 2013 by celogeek <me@celogeek.com>.
 
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
+This software is copyright (c) 2017 by Jens Rehsack.
+
+This is free software; you can redistribute it and/or modify it under the same terms as the Perl 5 programming language system itself.
 
 =cut
