@@ -1,62 +1,88 @@
-#
-# This file is part of MooX-Options
-#
-# This software is copyright (c) 2013 by celogeek <me@celogeek.com>.
-#
-# This is free software; you can redistribute it and/or modify it under
-# the same terms as the Perl 5 programming language system itself.
-#
 package MooX::Options::Descriptive::Usage;
 
-# ABSTRACT: Usage class
+use strict;
+use warnings FATAL => 'all';
+
+=head1 NAME
+
+MooX::Options::Descriptive::Usage - Usage class
+
+=head1 DESCRIPTION
+
+Usage class to display the error message.
+
+This class use the full size of your terminal
+
+=cut
 
 ## no critic (ProhibitExcessComplexity)
 
-use strict;
-use warnings;
+our $VERSION = "4.100";
 
-our $VERSION = '4.022';    # VERSION
-use feature 'say', 'state';
 use Getopt::Long::Descriptive;
+use Module::Runtime qw(use_module);
 use Scalar::Util qw/blessed/;
-use Locale::TextDomain 'MooX-Options';
+use Text::LineFold ();
 
-my %format_doc = (
-    's'  => __("String"),
-    's@' => __("[Strings]"),
-    'i'  => __("Int"),
-    'i@' => __("[Ints]"),
-    'o'  => __("Ext. Int"),
-    'o@' => __("[Ext. Ints]"),
-    'f'  => __("Real"),
-    'f@' => __("[Reals]"),
-);
+use Moo;
+with "MooX::Locale::Passthrough";
 
-sub _format_long_doc {
-    my $format          = shift;
-    my %format_long_doc = (
-        's'  => __("String"),
-        's@' => __("Array of Strings"),
-        'i'  => __("Integer"),
-        'i@' => __("Array of Integers"),
-        'o'  => __("Extended Integer"),
-        'o@' => __("Array of extended integers"),
-        'f'  => __("Real number"),
-        'f@' => __("Array of real numbers"),
-    );
-    return $format_long_doc{$format};
+has format_doc => ( is => "lazy" );
+
+## no critic (Subroutines::RequireFinalReturn, Subroutines::ProhibitUnusedPrivateSubroutines)
+
+sub _build_format_doc {
+    my $self = shift;
+    +{  's'  => $self->__("String"),
+        's@' => $self->__("[Strings]"),
+        'i'  => $self->__("Int"),
+        'i@' => $self->__("[Ints]"),
+        'o'  => $self->__("Ext. Int"),
+        'o@' => $self->__("[Ext. Ints]"),
+        'f'  => $self->__("Real"),
+        'f@' => $self->__("[Reals]"),
+    };
 }
 
-sub new {
-    my ( $class, $args ) = @_;
+has format_doc_long => ( is => "lazy" );
 
-    my %self;
-    @self{qw/options leader_text/} = @$args{qw/options leader_text/};
-
-    return bless \%self => $class;
+sub _build_format_doc_long {
+    my $self = shift;
+    +{  's'  => $self->__("String"),
+        's@' => $self->__("Array of Strings"),
+        'i'  => $self->__("Integer"),
+        'i@' => $self->__("Array of Integers"),
+        'o'  => $self->__("Extended Integer"),
+        'o@' => $self->__("Array of extended integers"),
+        'f'  => $self->__("Real number"),
+        'f@' => $self->__("Array of real numbers"),
+    };
 }
 
-sub leader_text { return shift->{leader_text} }
+=head1 ATTRIBUTES
+
+Following attributes are present and behave as GLD::Usage describe them.
+
+=head2 leader_text
+
+Text that appear on top of your message
+
+=head2 options
+
+The options spec of your message
+
+=cut
+
+has leader_text => ( is => "ro" );
+has options     => ( is => "ro" );
+
+=head1 METHODS
+
+=head2 sub_commands_text
+
+Return the list of sub commands if available.
+
+=cut
 
 sub sub_commands_text {
     my ($self) = @_;
@@ -70,10 +96,17 @@ sub sub_commands_text {
         $sub_commands = $sub_commands_options;
     }
     return if !@$sub_commands;
-    return "", __("SUB COMMANDS AVAILABLE: ") . join( ', ', @$sub_commands ),
-        "";
+    return "",
+        $self->__("SUB COMMANDS AVAILABLE: ")
+        . join( ', ', map { $_->{name} } @$sub_commands ), "";
 }
 
+=head2 text
+
+Return a compact help message.
+
+=cut
+
 sub text {
     my ($self) = @_;
     my %options_data
@@ -82,7 +115,7 @@ sub text {
         = defined $self->{target}
         ? $self->{target}->_options_config
         : ( spacer => " " );
-    my $getopt_options = $self->{options};
+    my $getopt_options = $self->options;
 
     my $lf = _get_line_fold();
 
@@ -97,7 +130,7 @@ sub text {
         }
         my ( $short, $format ) = $opt->{spec} =~ /(?:\|(\w))?(?:=(.*?))?$/x;
         my $format_doc_str;
-        $format_doc_str = $format_doc{$format} if defined $format;
+        $format_doc_str = $self->format_doc->{$format} if defined $format;
         $format_doc_str = 'JSON'
             if defined $options_data{ $opt->{name} }{json};
 
@@ -141,14 +174,19 @@ sub text {
 sub _get_line_fold {
     my $columns = $ENV{TEST_FORCE_COLUMN_SIZE}
         || eval {
-        require Term::Size::Any;
+        use_module("Term::Size::Any");
         [ Term::Size::Any::chars() ]->[0];
         } || 80;
 
-    require Text::LineFold;
     return Text::LineFold->new( ColMax => $columns - 4 );
 }
 
+=head2 option_help
+
+Return the help message for your options
+
+=cut
+
 sub option_help {
     my ($self) = @_;
     my %options_data
@@ -157,7 +195,7 @@ sub option_help {
         = defined $self->{target}
         ? $self->{target}->_options_config
         : ( spacer => " " );
-    my $getopt_options = $self->{options};
+    my $getopt_options = $self->options;
     my @message;
     my $lf = _get_line_fold();
     for my $opt (@$getopt_options) {
@@ -169,7 +207,7 @@ sub option_help {
         }
         my ( $short, $format ) = $opt->{spec} =~ /(?:\|(\w))?(?:=(.*?))?$/x;
         my $format_doc_str;
-        $format_doc_str = $format_doc{$format} if defined $format;
+        $format_doc_str = $self->format_doc->{$format} if defined $format;
         $format_doc_str = 'JSON'
             if defined $options_data{ $opt->{name} }{json};
         push @message,
@@ -185,8 +223,8 @@ sub option_help {
             "    ",
             "        ",
             defined $opt_data->{long_doc}
-            ? $opt_data->{long_doc}
-            : $opt->{desc}
+            ? $self->__( $opt_data->{long_doc} )
+            : $self->__( $opt->{desc} )
             );
     }
 
@@ -195,6 +233,12 @@ sub option_help {
         $self->sub_commands_text );
 }
 
+=head2 option_pod
+
+Return the usage message in pod format
+
+=cut
+
 sub option_pod {
     my ($self) = @_;
 
@@ -228,7 +272,7 @@ sub option_pod {
     push @man,
         (
         "=head1 SYNOPSIS",
-        $prog_name . " [-h] [" . __("long options ...") . "]"
+        $prog_name . " [-h] [" . $self->__("long options ...") . "]"
         );
 
     if ( defined( my $synopsis = $options_config{synopsis} ) ) {
@@ -238,7 +282,7 @@ sub option_pod {
     push @man, ( "=head1 OPTIONS", "=over" );
 
     my $spacer_escape = "E<" . ord( $options_config{spacer} ) . ">";
-    for my $opt ( @{ $self->{options} } ) {
+    for my $opt ( @{ $self->options } ) {
         if ( $opt->{desc} eq 'spacer' ) {
             push @man, "=back";
             push @man, $spacer_escape x 40;
@@ -247,7 +291,8 @@ sub option_pod {
         }
         my ( $short, $format ) = $opt->{spec} =~ /(?:\|(\w))?(?:=(.*?))?$/x;
         my $format_doc_str;
-        $format_doc_str = _format_long_doc($format) if defined $format;
+        $format_doc_str = $self->format_doc_long->{$format}
+            if defined $format;
         $format_doc_str = 'JSON'
             if defined $options_data{ $opt->{name} }{json};
 
@@ -262,8 +307,7 @@ sub option_pod {
 
         my $opt_data = $options_data{ $opt->{name} };
         $opt_data = {} if !defined $opt_data;
-        push @man,
-            defined $opt_data->{long_doc}
+        push @man, defined $opt_data->{long_doc}
             ? $opt_data->{long_doc}
             : $opt->{desc};
     }
@@ -273,14 +317,25 @@ sub option_pod {
         push @man, "=head1 AVAILABLE SUB COMMANDS";
         push @man, "=over";
         for my $sub_command (@$sub_commands) {
+            if ($sub_command->{command}->can("_options_config")
+                && defined(
+                    my $desc
+                        = { $sub_command->{command}->_options_config }
+                        ->{description}
+                )
+                )
+            {
+                push @man, "=item B<" . $sub_command->{name} . "> : " . $desc;
+            }
+            else {
+                push @man, "=item B<" . $sub_command->{name} . "> :";
+            }
+
             push @man,
-                (
-                "=item B<" . $sub_command . "> :",
-                $prog_name . " "
-                    . $sub_command
-                    . " [-h] ["
-                    . __("long options ...") . "]"
-                );
+                  $prog_name . " "
+                . $sub_command->{name}
+                . " [-h] ["
+                . $self->__("long options ...") . "]";
         }
         push @man, "=back";
     }
@@ -299,11 +354,17 @@ sub option_pod {
     return join( "\n\n", @man );
 }
 
+=head2 option_short_usage
+
+All options message without help
+
+=cut
+
 sub option_short_usage {
     my ($self) = @_;
     my %options_data
         = defined $self->{target} ? $self->{target}->_options_data : ();
-    my $getopt_options = $self->{options};
+    my $getopt_options = $self->options;
 
     my $prog_name = $self->{prog_name};
     $prog_name = Getopt::Long::Descriptive::prog_name if !defined $prog_name;
@@ -316,7 +377,7 @@ sub option_short_usage {
         }
         my ($format) = $opt->{spec} =~ /(?:\|\w)?(?:=(.*?))?$/x;
         my $format_doc_str;
-        $format_doc_str = $format_doc{$format} if defined $format;
+        $format_doc_str = $self->format_doc->{$format} if defined $format;
         $format_doc_str = 'JSON'
             if defined $options_data{ $opt->{name} }{json};
         push @message,
@@ -329,8 +390,20 @@ sub option_short_usage {
         join( " ", $prog_name, map { $_ eq '' ? " | " : "[ $_ ]" } @message );
 }
 
+=head2 warn
+
+Warn your options help message
+
+=cut
+
 sub warn { return CORE::warn shift->text }
 
+=head2 die
+
+Croak your options help message
+
+=cut
+
 sub die {
     my ($self) = @_;
     $self->{should_die} = 1;
@@ -345,87 +418,34 @@ use overload (
     }
 );
 
-1;
+=head1 SUPPORT
 
-__END__
+You can find documentation for this module with the perldoc command.
 
-=pod
+    perldoc MooX::Options
 
-=head1 NAME
+You can also look for information at:
 
-MooX::Options::Descriptive::Usage - Usage class
+=over 4
 
-=head1 VERSION
+=item * RT: CPAN's request tracker (report bugs here)
 
-version 4.022
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=MooX-Options>
 
-=head1 DESCRIPTION
+=item * AnnoCPAN: Annotated CPAN documentation
 
-Usage class to display the error message.
+L<http://annocpan.org/dist/MooX-Options>
 
-This class use the full size of your terminal
+=item * CPAN Ratings
 
-=head1 METHODS
+L<http://cpanratings.perl.org/d/MooX-Options>
 
-=head2 new
+=item * Search CPAN
 
-The object is create with L<MooX::Options::Descriptive>.
-
-Valid option is :
-
-=over
-
-=item leader_text
-
-Text that appear on top of your message
-
-=item options
-
-The options spec of your message
+L<http://search.cpan.org/dist/MooX-Options/>
 
 =back
 
-=head2 leader_text
-
-Return the leader_text.
-
-=head2 sub_commands_text
-
-Return the list of sub commands if available.
-
-=head2 text
-
-Return a compact help message.
-
-=head2 option_help
-
-Return the help message for your options
-
-=head2 option_pod
-
-Return the usage message in pod format
-
-=head2 option_short_usage
-
-All options message without help
-
-=head2 warn
-
-Warn your options help message
-
-=head2 die
-
-Croak your options help message
-
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website
-https://github.com/celogeek/MooX-Options/issues
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
 =head1 AUTHOR
 
 celogeek <me@celogeek.com>
@@ -434,7 +454,10 @@ celogeek <me@celogeek.com>
 
 This software is copyright (c) 2013 by celogeek <me@celogeek.com>.
 
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
+This software is copyright (c) 2017 by Jens Rehsack.
+
+This is free software; you can redistribute it and/or modify it under the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+1;
